import 'dart:developer';

import 'package:hub/features/backend/index.dart';
import 'package:hub/features/module/data/data_sources/index.dart';
import 'package:hub/features/module/domain/index.dart';
import 'package:hub/features/storage/index.dart';
import 'package:sqflite/sqflite.dart';

typedef LicenseStatus = bool;

class LicenseRepositoryImpl implements LicenseRepository {
  late final Database database;
  late final PhpGroup api;
  late final LicenseLocalDataSource localDataSource;
  late final LicenseRemoteDataSource remoteDataSource;

  LicenseRepositoryImpl([Database? database, PhpGroup? api]) {
    if (database != null) {
      this.database = database;
    } else {
      this.database = DatabaseService.database;
    }
    if (api != null) {
      this.api = api;
    } else {
      this.api = PhpGroup();
    }

    localDataSource = LicenseLocalDataSourceImpl(this.database);
    remoteDataSource = LicenseRemoteDataSourceImpl(this.api);
  }

  @override
  Future<bool> updateLicense() async {
    log('updateLicense');
    bool result = false;
    final bool isNewVersion = await localDataSource.isNewVersion();
    result = await localDataSource.setupLicense(isNewVersion);
    if (isNewVersion) {
      result = await remoteDataSource.fetchLicenses(isNewVersion);
    }

    return result;
  }

  @override
  Future<void> cleanLicense() async {
    return await remoteDataSource.cleanLicense();
  }

  @override
  Future<String?> getLicense(String key) async {
    final response = await localDataSource.get(key);
    return response;
  }

  @override
  Future<void> setLicense<T>(String key, T value) async {
    return await localDataSource.set(key, value);
  }
}
